/*
 * main.c
 *
 *  Created on: 27-12-2013
 *      Author: Mirosaw Karad
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

#define LED1 (1<<PB6)
#define LED2 (1<<PB7)
#define LED3 (1<<PD5)
#define LED4 (1<<PD6)
#define LED5 (1<<PD7)

#define LED1_ON PORTB &= ~LED1
#define LED1_OFF PORTB |= LED1
#define LED1_TOG PORTB ^= LED1
#define LED2_ON PORTB &= ~LED2
#define LED2_OFF PORTB |= LED2
#define LED2_TOG PORTB ^= LED2

#define LED3_ON PORTD &= ~LED3
#define LED3_OFF PORTD |= LED3
#define LED3_TOG PORTD ^= LED3
#define LED4_ON PORTD &= ~LED4
#define LED4_OFF PORTD |= LED4
#define LED4_TOG PORTD ^= LED4
#define LED5_ON PORTD &= ~LED5
#define LED5_OFF PORTD |= LED5
#define LED5_TOG PORTD ^= LED5

#define PCINT_K1 (1<<PC5)
#define PCINT_K2 (1<<PC4)
#define PCINT_K3 (1<<PC3)
#define PCINT_K4 (1<<PC2)
#define PCINT_K5 (1<<PC1)

#define BUZ (1<<PC0)

#define BUZ_OFF PORTC &= ~BUZ
#define BUZ_ON PORTC |= BUZ
#define BUZ_TOG PORTC ^= BUZ


// deklaracje funkcji
void leds_off(void);
void buz_start(void);
void buz_WAIT(void);
void buz_BAD(void);
void buz_TMO(void);
void buz_OK(void);

// definicje zmiennych globalnych
volatile uint8_t pressed;
uint16_t tmo;
uint8_t rnd;
uint8_t koniec=3;
uint8_t max_err;

// gwna funkcja programu
int main( void ) {

	DDRC |= (1<<PC0);  // pin buzzera jako wyjcie

	// konfiguracja diod LED
	DDRB |= LED1|LED2;
	DDRD |= LED3|LED4|LED5;
	leds_off();

	// podciganie linii klawiszy do VCC
	PORTC |= PCINT_K1|PCINT_K2|PCINT_K3|PCINT_K4|PCINT_K5;

	// konfiguracja PCINT1
	PCICR |= (1<<PCIE1);
	PCMSK1 |= PCINT_K1|PCINT_K2|PCINT_K3|PCINT_K4|PCINT_K5;


	sei();
	while(1) {

		koniec++;

		// inicjalizacja gry
		if( koniec > 3 ) {
			koniec=0;
			buz_WAIT();
			leds_off();
			_delay_ms(1000);
			pressed=0;
			max_err=0;
		}

		buz_start();	// odgos startu

		// losowanie diody
		while( 1 ) {
			rnd = rand();
			if( rnd < 5 ) break;
		}

		// zapalenie wylosowanej diody
		switch( rnd ) {
			case 0: LED1_ON; break;
			case 1: LED2_ON; break;
			case 2: LED3_ON; break;
			case 3: LED4_ON; break;
			case 4: LED5_ON; break;
		}

		tmo=200; // timeout = ok 0,5s
		while( !pressed ) {
			_delay_ms(1);
			if(tmo) tmo--;
			if(!tmo) break;
		}

		if( !tmo ) {		// jeli TimeOut
			buz_TMO();
		} else
		if( pressed != rnd+1 ) {	// jeli nie ten klawisz - jeli nie trafiony bandyta
			buz_BAD();
			if(koniec) koniec--;
			max_err++;
			if(max_err>6) {
				max_err=0;
				BUZ_ON;
				_delay_ms(1000);
				BUZ_OFF;
				_delay_ms(1000);
				BUZ_ON;
				_delay_ms(1000);
				BUZ_OFF;
				leds_off();
				koniec=3;
			}
		} else {			// jeli trafiony
			buz_OK();
			if( koniec ) koniec--;
//			if( max_err ) max_err--;
		}

		_delay_ms(1000);

		pressed=0;

	}
}




ISR( PCINT1_vect ) {
	if( !(PINC & PCINT_K1) ) pressed=1, LED1_OFF;
	if( !(PINC & PCINT_K2) ) pressed=2, LED2_OFF;
	if( !(PINC & PCINT_K3) ) pressed=3, LED3_OFF;
	if( !(PINC & PCINT_K4) ) pressed=4, LED4_OFF;
	if( !(PINC & PCINT_K5) ) pressed=5, LED5_OFF;

}

// ------- definicje funkcji
void leds_off(void) {
	LED1_OFF; LED2_OFF; LED3_OFF; LED4_OFF; LED5_OFF;
}


void buz_start(void) {
	uint8_t i;
	for(i=0; i<10; i++) _delay_ms(10), BUZ_TOG;
}

void buz_WAIT(void) {
	uint8_t a=1;
	while(1) {
		leds_off();
		switch(a) {
			case 1: LED1_TOG; break;
			case 2: LED2_TOG; break;
			case 3: LED3_TOG; break;
			case 4: LED4_TOG; break;
			case 5: LED5_TOG; break;
		}
		a++;
		if(a>5) a=1;
		_delay_ms(100);
		if(pressed) break;
	}
}

void buz_BAD(void) {
	uint8_t i;

	for(i=0; i<5; i++) {
		_delay_ms(200);
		BUZ_TOG;
		switch( pressed ) {
			case 1: LED1_TOG; break;
			case 2: LED2_TOG; break;
			case 3: LED3_TOG; break;
			case 4: LED4_TOG; break;
			case 5: LED5_TOG; break;
		}
		switch( rnd ) {
			case 0: LED1_TOG; break;
			case 1: LED2_TOG; break;
			case 2: LED3_TOG; break;
			case 3: LED4_TOG; break;
			case 4: LED5_TOG; break;
		}
	}
	BUZ_OFF;
	leds_off();
}

void buz_TMO(void) {
	uint8_t i;

	for(i=0; i<20; i++) {
		_delay_ms(100);
		BUZ_TOG;
		LED1_TOG; LED2_TOG; LED3_TOG; LED4_TOG; LED5_TOG;
	}
	leds_off();
}

void buz_OK(void) {
	uint8_t i;

	for(i=0; i<8; i++) {
		_delay_ms(20);
		BUZ_TOG;
		LED2_TOG; LED4_TOG;
	}
	leds_off();
}

